/*!
 * module deps
 */

const stream = require('stream')
const util = require('util')

/*!
 * globals
 */

const Readable = stream.Readable

/**
 * FirebaseReadStream constructor
 * created a Readable that read child at a firebase location and stream children
 *
 * @param { Firenext Query } query
 * @param { {pageSize = 100} } opts
 */

function FirebaseReadStream(query, opts) {
  if (!opts) opts = {}

  Readable.call(this, { objectMode: true })
  this.query = query
  this.pageSize = opts.pageSize || 100
  this.max = opts.max || Infinity
  this.count = 0
  this.marker = null
  this.reading = false
}

/*!
 * extend readable
 */

util.inherits(FirebaseReadStream, Readable)

/**
 * FirebaseReadStream internal _read
 * @see http://nodejs.org/api/stream.html#stream_readable_read_size_1
 */

FirebaseReadStream.prototype._read = function() {
  const stream = this
  var query = this.query

  if (stream.reading) return
  stream.reading = true

  if (this.marker) {
    query = query
      .startAt(this.marker.getPriority(), this.marker.key())
      .limitToFirst(this.pageSize + 1)
  } else {
    query = query
      .limitToFirst(this.pageSize)
  }

  query
    .exec()
    .then(function(data) {
      var pushResult, newMarker

      data.forEach(function(child) {
        if (!stream.marker || stream.marker.compareTo(child) < 0) {
          pushResult = stream.push(child)
          newMarker = child
          stream.count++
        }
      })

      // reset reading flag
      stream.reading = false
      stream.marker = newMarker

      // check if there is more items
      if (!newMarker) {
        stream.push(null)

      // check if we reached max
      } else if (stream.count >= stream.max) {
        stream.push(null)

      // read more if push returned true
      } else if (pushResult) {
        stream._read()
      }
    })
    .catch(function(error) {
      stream.emit('error', error)
    })
}

/*!
 * exports
 */

module.exports.FirebaseReadStream = FirebaseReadStream
module.exports = function firebaseReadStream(query, opts) {
  return new FirebaseReadStream(query, opts)
}
